@@ -11,6 +11,8 @@
 import logging
 
 from pcluster.constants import (
+    LUSTRE,
+    OPENZFS,
     PCLUSTER_IMAGE_BUILD_LOG_TAG,
     PCLUSTER_IMAGE_CONFIG_TAG,
     PCLUSTER_IMAGE_ID_TAG,
@@ -203,6 +205,10 @@ class InstanceTypeInfo:
 
         return vcpus
 
+    def instance_storage_supported(self) -> bool:
+        """Indicate whether instance storage is supported."""
+        return self.instance_type_data.get("InstanceStorageSupported")
+
     def supported_architecture(self):
         """Return the list of supported architectures."""
         supported_architectures = self.instance_type_data.get("ProcessorInfo").get("SupportedArchitectures")
@@ -218,10 +224,9 @@ class InstanceTypeInfo:
         return self.instance_type_data.get("InstanceType")
 
     def is_cpu_options_supported_in_lt(self):
-        """Check whether hyperthreading can be disabled via CPU options."""
-        # If default threads per core is 1, HT doesn't need to be disabled
+        """Check whether CPUOptions can be set in launch template."""
         # When CpuOptions is not supported, valid threads per core can't be found in vcpu info
-        return self.default_threads_per_core() > 1 and 1 in self.valid_threads_per_core()
+        return len(self.valid_threads_per_core()) != 0
 
     def is_ebs_optimized(self):
         """Check whether the instance has optimized EBS support."""
@@ -240,6 +245,10 @@ class InstanceTypeInfo:
             supported_classes.append("ondemand")
         return supported_classes
 
+    def ec2memory_size_in_mib(self):
+        """Return the amount of memory in MiB."""
+        return self.instance_type_data.get("MemoryInfo", {}).get("SizeInMiB")
+
 
 class FsxFileSystemInfo:
     """Data object wrapping the result of a describe_file_systems call."""
@@ -247,15 +256,41 @@ class FsxFileSystemInfo:
     def __init__(self, file_system_data):
         self.file_system_data = file_system_data
 
+    @property
+    def file_system_type(self):
+        """Return the type of FSx file system (LUSTRE, WINDOWS, ONTAP, or OPENZFS). WINDOWS is not supported."""
+        return self.file_system_data.get("FileSystemType")
+
     @property
     def mount_name(self):
-        """Return MountName of the filesystem."""
-        return self.file_system_data.get("LustreConfiguration").get("MountName")
+        """Return MountName of the FSx Lustre file system."""
+        return (
+            self.file_system_data.get("LustreConfiguration").get("MountName") if self.file_system_type == LUSTRE else ""
+        )
 
     @property
     def dns_name(self):
-        """Return DNSName of the filesystem."""
-        return self.file_system_data.get("DNSName")
+        """
+        Return DNSName of the file system.
+
+        Lustre, OpenZFS have DNS name on file systems. Ontap has DNS name on storage virtual machines.
+        """
+        return self.file_system_data.get("DNSName") if self.file_system_type in [LUSTRE, OPENZFS] else ""
+
+    @property
+    def file_system_id(self):
+        """Return id of the file system."""
+        return self.file_system_data.get("FileSystemId")
+
+    @property
+    def vpc_id(self):
+        """Return VPC id of the file system."""
+        return self.file_system_data.get("VpcId")
+
+    @property
+    def network_interface_ids(self):
+        """Return network interface ids of the file system."""
+        return self.file_system_data.get("NetworkInterfaceIds")
 
 
 class ImageInfo:
