@@ -10,13 +10,14 @@
 # limitations under the License.
 from enum import Enum
 
-from pcluster import utils
+from pcluster.config.cluster_config import QueueUpdateStrategy
+from pcluster.constants import DEFAULT_MAX_COUNT
 
 
-class UpdatePolicy(object):
+class UpdatePolicy:
     """Describes the policy that rules the update of a configuration parameter."""
 
-    class CheckResult(Enum):
+    class CheckResult(str, Enum):
         """Valid results for change checks."""
 
         SUCCEEDED = "SUCCEEDED"
@@ -26,12 +27,14 @@ class UpdatePolicy(object):
     def __init__(
         self,
         base_policy=None,
+        name=None,
         level=None,
         fail_reason=None,
         action_needed=None,
         condition_checker=None,
         print_succeeded=True,
     ):
+        self.name = None
         self.fail_reason = None
         self.action_needed = None
         self.condition_checker = None
@@ -39,11 +42,14 @@ class UpdatePolicy(object):
         self.level = 0
 
         if base_policy:
+            self.name = base_policy.name
             self.fail_reason = base_policy.fail_reason
             self.action_needed = base_policy.action_needed
             self.condition_checker = base_policy.condition_checker
             self.level = base_policy.level
 
+        if name:
+            self.name = name
         if level:
             self.level = level
         if fail_reason:
@@ -97,15 +103,61 @@ class UpdatePolicy(object):
         return self.fail_reason == other.fail_reason and self.level == other.level
 
 
+def actions_needed_queue_update_strategy(change, _):
+    actions = "Stop the compute fleet with the pcluster update-compute-fleet command"
+    # QueueUpdateStrategy can override UpdatePolicy of parameters under SlurmQueues
+    if is_slurm_queues_change(change):
+        actions += ", or set QueueUpdateStrategy in the configuration used for the 'update-cluster' operation"
+
+    return actions
+
+
+def condition_checker_compute_fleet_stop_on_remove(change, patch):
+    result = not patch.cluster.has_running_capacity()
+    # SlurmQueue or ComputeResource can be added but removal require compute fleet stop
+    if change.is_list and (is_slurm_queues_change(change) or change.key == "SlurmQueues"):
+        result = result or (change.old_value is None and change.new_value is not None)
+
+    return result
+
+
+def is_slurm_queues_change(change):
+    return any(path.startswith("SlurmQueues[") for path in change.path)
+
+
+def fail_reason_queue_update_strategy(change, _):
+    reason = "All compute nodes must be stopped"
+    # QueueUpdateStrategy can override UpdatePolicy of parameters under SlurmQueues
+    if is_slurm_queues_change(change):
+        reason += " or QueueUpdateStrategy must be set"
+
+    return reason
+
+
+def condition_checker_queue_update_strategy(change, patch):
+    result = not patch.cluster.has_running_capacity()
+    # QueueUpdateStrategy can override UpdatePolicy of parameters under SlurmQueues
+    if is_slurm_queues_change(change):
+        result = (
+            result
+            or patch.target_config.get("Scheduling")
+            .get("SlurmSettings", {})
+            .get("QueueUpdateStrategy", QueueUpdateStrategy.COMPUTE_FLEET_STOP.value)
+            != QueueUpdateStrategy.COMPUTE_FLEET_STOP.value
+        )
+
+    return result
+
+
 # Common fail_reason messages
 UpdatePolicy.FAIL_REASONS = {
     "ebs_volume_resize": "Updating the file system after a resize operation requires commands specific to your "
     "operating system.",
-    "ebs_sections_change": "EBS sections cannot be added or removed during a 'pcluster update' operation",
-    "extra_json_update": lambda change, patch: "Updating the extra_json parameter is not supported because it only "
-    "applies updates to compute nodes. If you still want to proceed, first stop the cluster with the "
-    "following command:\n{0} -c {1} {2} and then run an update with the --force flag".format(
-        "pcluster stop", patch.config_file, patch.cluster_name
+    "shared_storage_change": "Shared Storage cannot be added or removed during a 'pcluster update-cluster' operation",
+    "cookbook_update": lambda change, patch: (
+        "Updating cookbook related parameter is not supported because it only "
+        "applies updates to compute nodes. If you still want to proceed, first stop the compute fleet with the "
+        "pcluster update-compute-fleet command and then run an update with the --force-update flag"
     ),
 }
 
@@ -115,44 +167,15 @@ UpdatePolicy.ACTIONS_NEEDED = {
         "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/requesting-ebs-volume-modifications.html",
         "modify-ebs-volume",
     ),
-    "pcluster_stop": lambda change, patch: "Stop the cluster with the following command:\n{0} -c {1} {2}".format(
-        "pcluster stop", patch.config_file, patch.cluster_name
-    ),
+    "pcluster_stop": lambda change, patch: "Stop the compute fleet with the pcluster update-compute-fleet command",
+    "pcluster_stop_conditional": actions_needed_queue_update_strategy,
 }
 
-
-def _check_min_count(change, patch):
-    is_fleet_stopped = not utils.cluster_has_running_capacity(patch.stack_name)
-    if is_fleet_stopped:
-        return True
-
-    new_min, old_min = change.new_value, change.old_value
-    old_max = patch.base_config.get_section(change.section_key, change.section_label).get_param_value("max_count")
-    new_max = patch.target_config.get_section(change.section_key, change.section_label).get_param_value("max_count")
-    return new_min >= old_min and new_max - new_min >= old_max - old_min
-
-
-def _is_bucket_pcluster_generated(stack_name):
-    params = utils.get_stack(stack_name).get("Parameters")
-    return utils.get_cfn_param(params, "RemoveBucketOnDeletion") == "True"
-
-
-def _check_generated_bucket(change, patch):
-
-    # If bucket is generated (no cluster_resource_bucket specified when creating) and no change in config
-    # Old value is retrieve from CFN's ResourcesS3Bucket param, and new value is from config(not specified)
-    # Actual ResourcesS3Bucket will not change when updating, so no actual change when update is applied
-    # Since there is no change, user should not be informed of a change/diff
-    # Print no diff and proceed with updating other parameters
-    # Else display diff
-    # Inform user cluster_resource_bucket/ResourcesS3Bucket will not be updated even if force update
-    return _is_bucket_pcluster_generated(patch.stack_name) and not change.new_value
-
-
 # Base policies
 
 # Update is ignored
 UpdatePolicy.IGNORED = UpdatePolicy(
+    name="IGNORED",
     level=-10,
     fail_reason="-",
     condition_checker=(lambda change, patch: True),
@@ -161,135 +184,91 @@ UpdatePolicy.IGNORED = UpdatePolicy(
 )
 
 # Update supported
-UpdatePolicy.SUPPORTED = UpdatePolicy(level=0, fail_reason="-", condition_checker=(lambda change, patch: True))
+UpdatePolicy.SUPPORTED = UpdatePolicy(
+    name="SUPPORTED", level=0, fail_reason="-", condition_checker=(lambda change, patch: True)
+)
 
 # Checks resize of max_vcpus in Batch Compute Environment
 UpdatePolicy.AWSBATCH_CE_MAX_RESIZE = UpdatePolicy(
+    name="AWSBATCH_CE_MAX_RESIZE",
     level=1,
     fail_reason=lambda change, patch: "Max vCPUs can not be lower than the current Desired vCPUs ({0})".format(
-        utils.get_batch_ce_capacity(patch.stack_name)
+        patch.cluster.get_running_capacity()
     ),
     action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=lambda change, patch: utils.get_batch_ce_capacity(patch.stack_name)
-    <= patch.target_config.get_section("cluster").get_param_value("max_vcpus"),
-)
-
-UpdatePolicy.MAX_QUEUE_SIZE = UpdatePolicy(
-    level=1,
-    fail_reason="Shrinking the queue size requires the compute fleet to be stopped first",
-    action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=lambda change, patch: not utils.cluster_has_running_capacity(patch.stack_name)
-    or change.new_value >= change.old_value,
+    condition_checker=lambda change, patch: patch.cluster.get_running_capacity()
+    <= patch.target_config["Scheduling"]["AwsBatchQueues"][0]["ComputeResources"][0]["MaxvCpus"],
 )
 
 # Checks resize of max_count
 UpdatePolicy.MAX_COUNT = UpdatePolicy(
+    name="MAX_COUNT",
     level=1,
     fail_reason=lambda change, patch: "Shrinking a queue requires the compute fleet to be stopped first",
     action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=lambda change, patch: not utils.cluster_has_running_capacity(patch.stack_name)
-    or change.new_value >= change.old_value,
+    condition_checker=lambda change, patch: not patch.cluster.has_running_capacity()
+    or (change.new_value if change.new_value is not None else DEFAULT_MAX_COUNT)
+    >= (change.old_value if change.old_value is not None else DEFAULT_MAX_COUNT),
 )
 
-# Checks resize of min_count
-UpdatePolicy.MIN_COUNT = UpdatePolicy(
-    level=1,
-    fail_reason=lambda change, patch: "The applied change may cause existing nodes to be terminated hence requires "
-    "the compute fleet to be stopped first",
+# Update supported only with all compute nodes down or with replacement policy set different from COMPUTE_FLEET_STOP
+UpdatePolicy.QUEUE_UPDATE_STRATEGY = UpdatePolicy(
+    name="QUEUE_UPDATE_STRATEGY",
+    level=5,
+    fail_reason=fail_reason_queue_update_strategy,
+    action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop_conditional"],
+    condition_checker=condition_checker_queue_update_strategy,
+)
+
+# Update supported on new addition or on removal only with all compute nodes down
+UpdatePolicy.COMPUTE_FLEET_STOP_ON_REMOVE = UpdatePolicy(
+    name="COMPUTE_FLEET_STOP_ON_REMOVE",
+    level=7,
+    fail_reason="All compute nodes must be stopped",
     action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=_check_min_count,
-)
-
-# Checks that the value of the parameter has not been decreased
-UpdatePolicy.INCREASE_ONLY = UpdatePolicy(
-    level=2,
-    fail_reason=lambda change, patch: "Value of parameter '{0}' cannot be decreased".format(change.param_key),
-    action_needed=lambda change, patch: "Set the value of parameter '{0}' to '{1}' or greater".format(
-        change.param_key, change.old_value
-    ),
-    condition_checker=lambda change, patch: change.new_value >= change.old_value,
+    condition_checker=condition_checker_compute_fleet_stop_on_remove,
 )
 
 # Update supported only with all compute nodes down
 UpdatePolicy.COMPUTE_FLEET_STOP = UpdatePolicy(
+    name="COMPUTE_FLEET_STOP",
     level=10,
     fail_reason="All compute nodes must be stopped",
     action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=lambda change, patch: not utils.cluster_has_running_capacity(patch.stack_name),
+    condition_checker=lambda change, patch: not patch.cluster.has_running_capacity(),
 )
 
 # Update supported only with head node down
 UpdatePolicy.HEAD_NODE_STOP = UpdatePolicy(
+    name="HEAD_NODE_STOP",
     level=20,
     fail_reason="To perform this update action, the head node must be in a stopped state",
     action_needed=UpdatePolicy.ACTIONS_NEEDED["pcluster_stop"],
-    condition_checker=lambda change, patch: utils.get_head_node_state(patch.stack_name) == "stopped",
+    condition_checker=lambda change, patch: patch.cluster.head_node_instance.state == "stopped",
 )
 
 # Expected Behavior:
 # No bucket specified when create, no bucket specified when update: Display no diff, proceed with update
 # For all other cases: Display diff and block, value will not be updated even if forced
 UpdatePolicy.READ_ONLY_RESOURCE_BUCKET = UpdatePolicy(
+    name="READ_ONLY_RESOURCE_BUCKET",
     level=30,
     fail_reason=lambda change, patch: (
-        "'{0}' parameter is a read_only parameter that cannot be updated. "
+        "'{0}' parameter is a read only parameter that cannot be updated. "
         "New value '{1}' will be ignored and old value '{2}' will be used if you force the update."
-    ).format(
-        change.param_key,
-        change.new_value,
-        change.old_value,
-    ),
-    action_needed=lambda change, patch: "Restore the value of parameter '{0}' to '{1}'".format(
-        change.param_key, change.old_value
-    ),
-    condition_checker=_check_generated_bucket,
+    ).format(change.key, change.new_value, change.old_value),
+    action_needed=lambda change, patch: f"Restore the value of parameter '{change.key}' to '{change.old_value}'",
+    condition_checker=lambda change, patch: False,
     # We don't want to show the change if allowed (e.g local value is empty)
     print_succeeded=False,
 )
 
-
-def _get_fsx_options(stack_name):
-    params = utils.get_stack(stack_name).get("Parameters")
-    return utils.get_cfn_param(params, "FSXOptions").split(",")
-
-
-def _check_unmanaged_fsx(_, patch):
-    """
-    Return true if the cluster does not have a managed FSx.
-
-    We have an unmanaged FSx if both shared_dir and fsx_fs_id are specified.
-    """
-    # FSXOptions is a comma separated string of the following items: shared_dir,fsx_fs_id,... (see CFN template)
-    fsx_options = _get_fsx_options(patch.stack_name)
-    fsx_shared_dir = fsx_options[0]
-    fsx_fs_id = fsx_options[1]
-
-    return fsx_shared_dir != "NONE" and fsx_fs_id != "NONE"
-
-
-# Expected Behavior:
-# There is a managed FSx specified at cluster creation time, vpc security group is modified when updating: block update
-# For all other cases the change is supported.
-UpdatePolicy.MANAGED_FSX = UpdatePolicy(
-    level=40,
-    fail_reason=lambda change, patch: (
-        "'{0}' parameter cannot be updated when a managed FSx for Lustre file system is configured. "
-        "Do not force the update or the file system will be deleted and the cluster enter an unrecoverable state."
-    ).format(change.param_key),
-    action_needed=lambda change, patch: (
-        "Restore the value of parameter '{0}' to '{1}'".format(change.param_key, change.old_value)
-        if change.old_value is not None
-        else "Remove the parameter '{0}'".format(change.param_key)
-    ),
-    condition_checker=_check_unmanaged_fsx,
-    print_succeeded=True,
-)
-
 # Update effects are unknown.
 # WARNING: This is the default value for all new configuration parameters. All parameters must be linked to a specific
 # update policy instead of UNKNOWN to pass unit tests.
 #
 UpdatePolicy.UNKNOWN = UpdatePolicy(
+    name="UNKNOWN",
     level=100,
     fail_reason="Update currently not supported",
     action_needed="Restore the previous parameter value for the unsupported changes.",
@@ -297,14 +276,13 @@ UpdatePolicy.UNKNOWN = UpdatePolicy(
 
 # Update not supported
 UpdatePolicy.UNSUPPORTED = UpdatePolicy(
+    name="UNSUPPORTED",
     level=1000,
-    fail_reason=lambda change, patch: "Update actions are not currently supported for the '{0}' parameter".format(
-        change.param_key
-    ),
+    fail_reason=lambda change, patch: (f"Update actions are not currently supported for the '{change.key}' parameter"),
     action_needed=lambda change, patch: (
-        "Restore '{0}' value to '{1}'".format(change.param_key, change.old_value)
+        f"Restore '{change.key}' value to '{change.old_value}'"
         if change.old_value is not None
-        else "Remove the parameter '{0}'".format(change.param_key)
+        else "{0} the parameter '{1}'".format("Restore" if change.is_list else "Remove", change.key)
     )
     + ". If you need this change, please consider creating a new cluster instead of updating the existing one.",
 )
