@@ -55,7 +55,8 @@ function error_exit
 {
   # wait logs flush before signaling the failure
   sleep 10
-  cfn-signal --exit-code=1 --reason="$1" "${!wait_condition_handle_presigned_url}"
+  reason=$(cat /var/log/parallelcluster/chef_error_msg 2>/dev/null) || reason="$1"
+  cfn-signal --exit-code=1 --reason="${!reason}" "${!wait_condition_handle_presigned_url}"
   exit 1
 }
 function vendor_cookbook
@@ -75,6 +76,9 @@ function vendor_cookbook
 
 # deploy config files
 export PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/opt/aws/bin
+# Load ParallelCluster environment variables
+[ -f /etc/profile.d/pcluster.sh ] && . /etc/profile.d/pcluster.sh
+
 cd /tmp
 cfn-init -s ${AWS::StackName} -v -c deployFiles -r HeadNodeLaunchTemplate --region ${AWS::Region}
 wait_condition_handle_presigned_url=$(cat /tmp/wait_condition_handle.txt)
