@@ -15,6 +15,7 @@
 #
 
 import copy
+import hashlib
 import logging
 import re
 from urllib.request import urlopen
@@ -24,6 +25,7 @@ from marshmallow import ValidationError, fields, post_load, pre_dump, pre_load,
 from yaml import YAMLError
 
 from pcluster.aws.aws_api import AWSApi
+from pcluster.aws.common import AWSClientError
 from pcluster.config.cluster_config import (
     AdditionalPackages,
     AmiSearchFilters,
@@ -47,6 +49,8 @@ from pcluster.config.cluster_config import (
     Dns,
     Efa,
     EphemeralVolume,
+    ExistingFsxOntap,
+    ExistingFsxOpenZfs,
     HeadNode,
     HeadNodeImage,
     HeadNodeNetworking,
@@ -60,6 +64,7 @@ from pcluster.config.cluster_config import (
     PlacementGroup,
     Proxy,
     QueueImage,
+    QueueUpdateStrategy,
     Raid,
     Roles,
     RootVolume,
@@ -88,7 +93,7 @@ from pcluster.config.cluster_config import (
     SchedulerPluginUser,
     SharedEbs,
     SharedEfs,
-    SharedFsx,
+    SharedFsxLustre,
     SlurmClusterConfig,
     SlurmComputeResource,
     SlurmQueue,
@@ -96,14 +101,20 @@ from pcluster.config.cluster_config import (
     SlurmScheduling,
     SlurmSettings,
     Ssh,
+    SudoerConfiguration,
+    Timeouts,
 )
 from pcluster.config.update_policy import UpdatePolicy
 from pcluster.constants import (
     DELETION_POLICIES,
     DELETION_POLICIES_WITH_SNAPSHOT,
-    EBS_VOLUME_SIZE_DEFAULT,
-    FSX_HDD_THROUGHPUT,
-    FSX_SSD_THROUGHPUT,
+    FSX_LUSTRE,
+    FSX_ONTAP,
+    FSX_OPENZFS,
+    FSX_VOLUME_ID_REGEX,
+    LUSTRE,
+    ONTAP,
+    OPENZFS,
     SCHEDULER_PLUGIN_MAX_NUMBER_OF_USERS,
     SUPPORTED_OSES,
 )
@@ -138,7 +149,7 @@ class HeadNodeRootVolumeSchema(BaseSchema):
             )
         },
     )
-    iops = fields.Int(metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    iops = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     size = fields.Int(
         metadata={
             "update_policy": UpdatePolicy(
@@ -148,52 +159,39 @@ class HeadNodeRootVolumeSchema(BaseSchema):
             )
         }
     )
-    throughput = fields.Int(metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    throughput = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     encrypted = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
-    delete_on_termination = fields.Bool(metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    delete_on_termination = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate resource."""
         return RootVolume(**data)
 
-    @validates("size")
-    def validate_size(self, value):
-        """Validate the size of root volume."""
-        if value < EBS_VOLUME_SIZE_DEFAULT:
-            raise ValidationError(
-                f"Root volume size {value} is invalid. It must be at least {EBS_VOLUME_SIZE_DEFAULT}."
-            )
-
 
 class QueueRootVolumeSchema(BaseSchema):
     """Represent the RootVolume schema for the queue."""
 
-    size = fields.Int(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    encrypted = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    volume_type = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    iops = fields.Int(metadata={"update_policy": UpdatePolicy.SUPPORTED})
-    throughput = fields.Int(metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    size = fields.Int(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    encrypted = fields.Bool(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    volume_type = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    iops = fields.Int(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    throughput = fields.Int(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
         """Generate resource."""
         return RootVolume(**data)
 
-    @validates("size")
-    def validate_size(self, value):
-        """Validate the size of root volume."""
-        if value < EBS_VOLUME_SIZE_DEFAULT:
-            raise ValidationError(
-                f"Root volume size {value} is invalid. It must be at least {EBS_VOLUME_SIZE_DEFAULT}."
-            )
-
 
 class RaidSchema(BaseSchema):
     """Represent the schema of the parameters specific to Raid. It is a child of EBS schema."""
 
     raid_type = fields.Int(
-        data_key="Type", validate=validate.OneOf([0, 1]), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        required=True,
+        data_key="Type",
+        validate=validate.OneOf([0, 1]),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     number_of_volumes = fields.Int(
         validate=validate.Range(min=2, max=5), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
@@ -260,7 +258,7 @@ class QueueEphemeralVolumeSchema(BaseSchema):
     """Represent the schema of ephemeral volume.It is a child of storage schema."""
 
     mount_dir = fields.Str(
-        validate=get_field_validator("file_path"), metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+        validate=get_field_validator("file_path"), metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
     )
 
     @post_load
@@ -286,9 +284,9 @@ class HeadNodeStorageSchema(BaseSchema):
 class QueueStorageSchema(BaseSchema):
     """Represent the schema of storage attached to a node."""
 
-    root_volume = fields.Nested(QueueRootVolumeSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    root_volume = fields.Nested(QueueRootVolumeSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
     ephemeral_volume = fields.Nested(
-        QueueEphemeralVolumeSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+        QueueEphemeralVolumeSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
     )
 
     @post_load
@@ -343,7 +341,7 @@ class FsxLustreSettingsSchema(BaseSchema):
 
     storage_capacity = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     deployment_type = fields.Str(
-        validate=validate.OneOf(["SCRATCH_1", "SCRATCH_2", "PERSISTENT_1"]),
+        validate=validate.OneOf(["SCRATCH_1", "SCRATCH_2", "PERSISTENT_1", "PERSISTENT_2"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     imported_file_chunk_size = fields.Int(
@@ -363,10 +361,7 @@ class FsxLustreSettingsSchema(BaseSchema):
     daily_automatic_backup_start_time = fields.Str(
         validate=validate.Regexp(r"^([01]\d|2[0-3]):([0-5]\d)$"), metadata={"update_policy": UpdatePolicy.SUPPORTED}
     )
-    per_unit_storage_throughput = fields.Int(
-        validate=validate.OneOf(FSX_SSD_THROUGHPUT + FSX_HDD_THROUGHPUT),
-        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
-    )
+    per_unit_storage_throughput = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     backup_id = fields.Str(
         validate=validate.Regexp("^(backup-[0-9a-f]{8,})$"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
@@ -426,6 +421,26 @@ class FsxLustreSettingsSchema(BaseSchema):
                 raise ValidationError(message=messages)
 
 
+class FsxOpenZfsSettingsSchema(BaseSchema):
+    """Represent the FSX OpenZFS schema."""
+
+    volume_id = fields.Str(
+        required=True,
+        validate=validate.Regexp(FSX_VOLUME_ID_REGEX),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+    )
+
+
+class FsxOntapSettingsSchema(BaseSchema):
+    """Represent the FSX Ontap schema."""
+
+    volume_id = fields.Str(
+        required=True,
+        validate=validate.Regexp(FSX_VOLUME_ID_REGEX),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+    )
+
+
 class SharedStorageSchema(BaseSchema):
     """Represent the generic SharedStorage schema."""
 
@@ -435,17 +450,25 @@ class SharedStorageSchema(BaseSchema):
     name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     storage_type = fields.Str(
         required=True,
-        validate=validate.OneOf(["Ebs", "FsxLustre", "Efs"]),
+        validate=validate.OneOf(["Ebs", FSX_LUSTRE, FSX_OPENZFS, FSX_ONTAP, "Efs"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     ebs_settings = fields.Nested(EbsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     efs_settings = fields.Nested(EfsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     fsx_lustre_settings = fields.Nested(FsxLustreSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    fsx_open_zfs_settings = fields.Nested(
+        FsxOpenZfsSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    fsx_ontap_settings = fields.Nested(FsxOntapSettingsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     @validates_schema
     def no_coexist_storage_settings(self, data, **kwargs):
         """Validate that *_settings for different storage types do not co-exist."""
-        if self.fields_coexist(data, ["ebs_settings", "efs_settings", "fsx_lustre_settings"], **kwargs):
+        if self.fields_coexist(
+            data,
+            ["ebs_settings", "efs_settings", "fsx_lustre_settings", "fsx_open_zfs_settings", "fsx_ontap_settings"],
+            **kwargs,
+        ):
             raise ValidationError("Multiple *Settings sections cannot be specified in the SharedStorage items.")
 
     @validates_schema
@@ -454,7 +477,9 @@ class SharedStorageSchema(BaseSchema):
         for storage_type, settings in [
             ("Ebs", "ebs_settings"),
             ("Efs", "efs_settings"),
-            ("FsxLustre", "fsx_lustre_settings"),
+            (FSX_LUSTRE, "fsx_lustre_settings"),
+            (FSX_OPENZFS, "fsx_open_zfs_settings"),
+            (FSX_ONTAP, "fsx_ontap_settings"),
         ]:
             # Verify the settings section is associated to the right storage type
             if data.get(settings, None) and storage_type != data.get("storage_type"):
@@ -469,16 +494,24 @@ class SharedStorageSchema(BaseSchema):
         storage_type = data.get("storage_type")
         shared_volume_attributes = {"mount_dir": data.get("mount_dir"), "name": data.get("name")}
         settings = (
-            data.get("efs_settings", None) or data.get("fsx_lustre_settings", None) or data.get("ebs_settings", None)
+            data.get("efs_settings", None)
+            or data.get("ebs_settings", None)
+            or data.get("fsx_lustre_settings", None)
+            or data.get("fsx_open_zfs_settings", None)
+            or data.get("fsx_ontap_settings", None)
         )
         if settings:
             shared_volume_attributes.update(**settings)
         if storage_type == "Efs":
             return SharedEfs(**shared_volume_attributes)
-        elif storage_type == "FsxLustre":
-            return SharedFsx(**shared_volume_attributes)
         elif storage_type == "Ebs":
             return SharedEbs(**shared_volume_attributes)
+        elif storage_type == FSX_LUSTRE:
+            return SharedFsxLustre(**shared_volume_attributes)
+        elif storage_type == FSX_OPENZFS:
+            return ExistingFsxOpenZfs(**shared_volume_attributes)
+        elif storage_type == FSX_ONTAP:
+            return ExistingFsxOntap(**shared_volume_attributes)
         return None
 
     @pre_dump
@@ -489,14 +522,17 @@ class SharedStorageSchema(BaseSchema):
         if adapted_data.shared_storage_type == "efs":
             storage_type = "efs"
         elif adapted_data.shared_storage_type == "fsx":
-            storage_type = "fsx_lustre"
+            mapping = {LUSTRE: "fsx_lustre", OPENZFS: "fsx_open_zfs", ONTAP: "fsx_ontap"}
+            storage_type = mapping.get(adapted_data.file_system_type)
         else:  # "raid", "ebs"
             storage_type = "ebs"
         setattr(adapted_data, f"{storage_type}_settings", copy.copy(adapted_data))
         # Restore storage type attribute
-        adapted_data.storage_type = (
-            "FsxLustre" if adapted_data.shared_storage_type == "fsx" else storage_type.capitalize()
-        )
+        if adapted_data.shared_storage_type == "fsx":
+            mapping = {LUSTRE: FSX_LUSTRE, OPENZFS: FSX_OPENZFS, ONTAP: FSX_ONTAP}
+            adapted_data.storage_type = mapping.get(adapted_data.file_system_type)
+        else:
+            adapted_data.storage_type = storage_type.capitalize()
         return adapted_data
 
     @validates("mount_dir")
@@ -527,7 +563,7 @@ class HeadNodeProxySchema(BaseSchema):
 class QueueProxySchema(BaseSchema):
     """Represent the schema of proxy for a queue."""
 
-    http_proxy_address = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    http_proxy_address = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -572,8 +608,8 @@ class HeadNodeNetworkingSchema(BaseNetworkingSchema):
 class PlacementGroupSchema(BaseSchema):
     """Represent the schema of placement group."""
 
-    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    id = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    id = fields.Str(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -588,7 +624,7 @@ class QueueNetworkingSchema(BaseNetworkingSchema):
         fields.Str(validate=get_field_validator("subnet_id")),
         required=True,
         validate=validate.Length(equal=1),
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+        metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY},
     )
     assign_public_ip = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
@@ -596,8 +632,10 @@ class QueueNetworkingSchema(BaseNetworkingSchema):
 class SlurmQueueNetworkingSchema(QueueNetworkingSchema):
     """Represent the schema of the Networking, child of slurm Queue."""
 
-    placement_group = fields.Nested(PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    proxy = fields.Nested(QueueProxySchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    placement_group = fields.Nested(
+        PlacementGroupSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
+    proxy = fields.Nested(QueueProxySchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -651,8 +689,8 @@ class DcvSchema(BaseSchema):
 class EfaSchema(BaseSchema):
     """Represent the schema of EFA for a Compute Resource."""
 
-    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    gdr_support = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    gdr_support = fields.Bool(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -819,7 +857,8 @@ class QueueIamSchema(IamSchema):
     """Represent the schema of IAM for Queue."""
 
     instance_profile = fields.Str(
-        metadata={"update_policy": UpdatePolicy.SUPPORTED}, validate=validate.Regexp("^arn:.*:instance-profile/")
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+        validate=validate.Regexp("^arn:.*:instance-profile/"),
     )
 
 
@@ -870,12 +909,29 @@ class AmiSearchFiltersSchema(BaseSchema):
         return AmiSearchFilters(**data)
 
 
+class TimeoutsSchema(BaseSchema):
+    """Represent the schema of the Timeouts section."""
+
+    head_node_bootstrap_timeout = fields.Int(
+        validate=validate.Range(min=1), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    compute_node_bootstrap_timeout = fields.Int(
+        validate=validate.Range(min=1), metadata={"update_policy": UpdatePolicy.SUPPORTED}
+    )
+
+    @post_load()
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return Timeouts(**data)
+
+
 class ClusterDevSettingsSchema(BaseDevSettingsSchema):
     """Represent the schema of Dev Setting."""
 
     cluster_template = fields.Str(metadata={"update_policy": UpdatePolicy.SUPPORTED})
     ami_search_filters = fields.Nested(AmiSearchFiltersSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     instance_types_data = fields.Str(metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    timeouts = fields.Nested(TimeoutsSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -922,7 +978,7 @@ class QueueImageSchema(BaseSchema):
 
     custom_ami = fields.Str(
         validate=validate.Regexp(r"^ami-[0-9a-z]{8}$|^ami-[0-9a-z]{17}$"),
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+        metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY},
     )
 
     @post_load
@@ -958,8 +1014,8 @@ class HeadNodeCustomActionsSchema(BaseSchema):
 class QueueCustomActionSchema(BaseSchema):
     """Represent the schema of the custom action."""
 
-    script = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    args = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    script = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
+    args = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -970,9 +1026,11 @@ class QueueCustomActionSchema(BaseSchema):
 class QueueCustomActionsSchema(BaseSchema):
     """Represent the schema for all available custom actions."""
 
-    on_node_start = fields.Nested(QueueCustomActionSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    on_node_start = fields.Nested(
+        QueueCustomActionSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
     on_node_configured = fields.Nested(
-        QueueCustomActionSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+        QueueCustomActionSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
     )
 
     @post_load
@@ -994,7 +1052,7 @@ class HeadNodeSchema(BaseSchema):
     dcv = fields.Nested(DcvSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     custom_actions = fields.Nested(HeadNodeCustomActionsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     iam = fields.Nested(HeadNodeIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
-    imds = fields.Nested(ImdsSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    imds = fields.Nested(ImdsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     image = fields.Nested(HeadNodeImageSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     @post_load()
@@ -1015,9 +1073,12 @@ class SlurmComputeResourceSchema(_ComputeResourceSchema):
     instance_type = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
     max_count = fields.Int(validate=validate.Range(min=1), metadata={"update_policy": UpdatePolicy.MAX_COUNT})
     min_count = fields.Int(validate=validate.Range(min=0), metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    spot_price = fields.Float(validate=validate.Range(min=0), metadata={"update_policy": UpdatePolicy.SUPPORTED})
-    efa = fields.Nested(EfaSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    spot_price = fields.Float(
+        validate=validate.Range(min=0), metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
+    efa = fields.Nested(EfaSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
     disable_simultaneous_multithreading = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    schedulable_memory = fields.Int(metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1066,7 +1127,7 @@ class SchedulerPluginComputeResourceSchema(SlurmComputeResourceSchema):
 class ComputeSettingsSchema(BaseSchema):
     """Represent the schema of the compute_settings schedulers queues."""
 
-    local_storage = fields.Nested(QueueStorageSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    local_storage = fields.Nested(QueueStorageSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
     @post_load()
     def make_resource(self, data, **kwargs):
@@ -1080,19 +1141,21 @@ class BaseQueueSchema(BaseSchema):
     name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     capacity_type = fields.Str(
         validate=validate.OneOf([event.value for event in CapacityType]),
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+        metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY},
     )
 
 
 class _CommonQueueSchema(BaseQueueSchema):
     """Represent the schema of common part between Slurm and Scheduler Plugin Queue."""
 
-    compute_settings = fields.Nested(ComputeSettingsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    compute_settings = fields.Nested(
+        ComputeSettingsSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
+    )
     custom_actions = fields.Nested(
-        QueueCustomActionsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+        QueueCustomActionsSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
     )
     iam = fields.Nested(QueueIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
-    image = fields.Nested(QueueImageSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    image = fields.Nested(QueueImageSchema, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY})
 
 
 class SlurmQueueSchema(_CommonQueueSchema):
@@ -1101,10 +1164,10 @@ class SlurmQueueSchema(_CommonQueueSchema):
     compute_resources = fields.Nested(
         SlurmComputeResourceSchema,
         many=True,
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP_ON_REMOVE, "update_key": "Name"},
     )
     networking = fields.Nested(
-        SlurmQueueNetworkingSchema, required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+        SlurmQueueNetworkingSchema, required=True, metadata={"update_policy": UpdatePolicy.QUEUE_UPDATE_STRATEGY}
     )
 
     @post_load
@@ -1168,7 +1231,12 @@ class SlurmSettingsSchema(BaseSchema):
     """Represent the schema of the Scheduling Settings."""
 
     scaledown_idletime = fields.Int(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    dns = fields.Nested(DnsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    dns = fields.Nested(DnsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    queue_update_strategy = fields.Str(
+        validate=validate.OneOf([strategy.value for strategy in QueueUpdateStrategy]),
+        metadata={"update_policy": UpdatePolicy.IGNORED},
+    )
+    enable_memory_based_scheduling = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1236,7 +1304,9 @@ class SchedulerPluginRequirementsSchema(BaseSchema):
     supports_cluster_update = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     supported_parallel_cluster_versions = fields.Str(
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
-        validate=validate.Regexp(r"^((>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+,\s*)*(>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+$"),
+        validate=validate.Regexp(
+            r"^((>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+([a-z][0-9]+)?,\s*)*(>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+([a-z][0-9]+)?$"
+        ),
     )
 
     @post_load
@@ -1249,6 +1319,12 @@ class SchedulerPluginCloudFormationClusterInfrastructureSchema(BaseSchema):
     """Represent the CloudFormation section of the Scheduler Plugin ClusterInfrastructure schema."""
 
     template = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    s3_bucket_owner = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^\d{12}$")
+    )
+    checksum = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^[A-Fa-f0-9]{64}$")
+    )
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1275,6 +1351,12 @@ class SchedulerPluginClusterSharedArtifactSchema(BaseSchema):
     """Represent the schema for Cluster Shared Artifact in a Scheduler Plugin."""
 
     source = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    s3_bucket_owner = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^\d{12}$")
+    )
+    checksum = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^[A-Fa-f0-9]{64}$")
+    )
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1334,10 +1416,7 @@ class SchedulerPluginEventsSchema(BaseSchema):
     head_cluster_update = fields.Nested(
         SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
-    head_compute_fleet_start = fields.Nested(
-        SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
-    )
-    head_compute_fleet_stop = fields.Nested(
+    head_compute_fleet_update = fields.Nested(
         SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
 
@@ -1394,11 +1473,26 @@ class SchedulerPluginMonitoringSchema(BaseSchema):
         return SchedulerPluginMonitoring(**data)
 
 
+class SudoerConfigurationSchema(BaseSchema):
+    """Represent the SudoerConfiguration for scheduler plugin SystemUsers declared in the SchedulerDefinition."""
+
+    commands = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    run_as = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SudoerConfiguration(**data)
+
+
 class SchedulerPluginUserSchema(BaseSchema):
     """Represent the schema of the Scheduler Plugin."""
 
     name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     enable_imds = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    sudoer_configuration = fields.Nested(
+        SudoerConfigurationSchema, many=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Name"}
+    )
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1410,9 +1504,11 @@ class SchedulerPluginDefinitionSchema(BaseSchema):
     """Represent the schema of the Scheduler Plugin SchedulerDefinition."""
 
     plugin_interface_version = fields.Str(
-        required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.OneOf(["1.0"])
+        required=True,
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+        validate=validate.Regexp(r"^[0-9]+\.[0-9]+$"),
     )
-    metadata = fields.Dict(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    metadata = fields.Dict(metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, required=True)
     requirements = fields.Nested(
         SchedulerPluginRequirementsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
@@ -1441,6 +1537,13 @@ class SchedulerPluginDefinitionSchema(BaseSchema):
         """Generate resource."""
         return SchedulerPluginDefinition(**data)
 
+    @validates("metadata")
+    def validate_metadata(self, value):
+        """Validate metadata contains fieds 'name' and 'version'."""
+        for key in ["Name", "Version"]:
+            if key not in value.keys():
+                raise ValidationError(f"{key} is required for scheduler plugin Metadata.")
+
 
 class SchedulerPluginSettingsSchema(BaseSchema):
     """Represent the schema of the Scheduling Settings."""
@@ -1450,6 +1553,93 @@ class SchedulerPluginSettingsSchema(BaseSchema):
     )
     grant_sudo_privileges = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     custom_settings = fields.Dict(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    scheduler_definition_s3_bucket_owner = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^\d{12}$")
+    )
+    scheduler_definition_checksum = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^[A-Fa-f0-9]{64}$")
+    )
+
+    def _verify_checksum(self, file_content, original_definition, expected_checksum):
+        if expected_checksum:
+            actual_checksum = hashlib.sha256(file_content.encode()).hexdigest()
+            if actual_checksum != expected_checksum:
+                raise ValidationError(
+                    f"Error when validating SchedulerDefinition '{original_definition}': "
+                    f"checksum ({actual_checksum}) does not match expected one ({expected_checksum})"
+                )
+
+    def _fetch_scheduler_definition_from_s3(self, original_scheduler_definition, s3_bucket_owner):
+        try:
+            bucket_parsing_result = parse_bucket_url(original_scheduler_definition)
+            result = AWSApi.instance().s3.get_object(
+                bucket_name=bucket_parsing_result["bucket_name"],
+                key=bucket_parsing_result["object_key"],
+                expected_bucket_owner=s3_bucket_owner,
+            )
+            scheduler_definition = result["Body"].read().decode("utf-8")
+            return scheduler_definition
+        except AWSClientError as e:
+            error_message = (
+                f"Error while downloading scheduler definition from {original_scheduler_definition}: {str(e)}"
+            )
+            if s3_bucket_owner and e.error_code == "AccessDenied":
+                error_message = (
+                    f"{error_message}. This can be due to bucket owner not matching the expected "
+                    f"one '{s3_bucket_owner}'"
+                )
+            raise ValidationError(error_message) from e
+        except Exception as e:
+            raise ValidationError(
+                f"Error while downloading scheduler definition from {original_scheduler_definition}: {str(e)}"
+            ) from e
+
+    def _fetch_scheduler_definition_from_https(self, original_scheduler_definition):
+        try:
+            with urlopen(original_scheduler_definition) as f:  # nosec nosemgrep
+                scheduler_definition = f.read().decode("utf-8")
+                return scheduler_definition
+        except Exception:
+            error_message = (
+                f"Error while downloading scheduler definition from {original_scheduler_definition}: "
+                "The provided URL is invalid or unavailable."
+            )
+            raise ValidationError(error_message)
+
+    def _validate_scheduler_definition_url(self, original_scheduler_definition, s3_bucket_owner):
+        """Validate SchedulerDefinition url is valid."""
+        if not original_scheduler_definition.startswith("s3") and not original_scheduler_definition.startswith("https"):
+            raise ValidationError(
+                f"Error while downloading scheduler definition from {original_scheduler_definition}: The provided value"
+                " for SchedulerDefinition is invalid. You can specify this as an S3 URL, HTTPS URL or as an inline "
+                "YAML object."
+            )
+        if original_scheduler_definition.startswith("https") and s3_bucket_owner:
+            raise ValidationError(
+                f"Error while downloading scheduler definition from {original_scheduler_definition}: "
+                "SchedulerDefinitionS3BucketOwner can only be specified when SchedulerDefinition is S3 URL."
+            )
+
+    def _fetch_scheduler_definition_from_url(
+        self, original_scheduler_definition, s3_bucket_owner, scheduler_definition_checksum, data
+    ):
+        LOGGER.info("Downloading scheduler plugin definition from %s", original_scheduler_definition)
+        if original_scheduler_definition.startswith("s3"):
+            scheduler_definition = self._fetch_scheduler_definition_from_s3(
+                original_scheduler_definition, s3_bucket_owner
+            )
+        elif original_scheduler_definition.startswith("https"):
+            scheduler_definition = self._fetch_scheduler_definition_from_https(original_scheduler_definition)
+
+        self._verify_checksum(scheduler_definition, original_scheduler_definition, scheduler_definition_checksum)
+
+        LOGGER.info("Using the following scheduler plugin definition:\n%s", scheduler_definition)
+        try:
+            data["SchedulerDefinition"] = yaml.safe_load(scheduler_definition)
+        except YAMLError as e:
+            raise ValidationError(
+                f"The retrieved SchedulerDefinition ({original_scheduler_definition}) is not a valid YAML."
+            ) from e
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1460,36 +1650,18 @@ class SchedulerPluginSettingsSchema(BaseSchema):
     def fetch_scheduler_definition(self, data, **kwargs):
         """Fetch scheduler definition if it is s3 or https url."""
         original_scheduler_definition = data["SchedulerDefinition"]
+        s3_bucket_owner = data.get("SchedulerDefinitionS3BucketOwner", None)
+        scheduler_definition_checksum = data.get("SchedulerDefinitionChecksum", None)
         if isinstance(original_scheduler_definition, str):
-            LOGGER.info("Downloading scheduler plugin definition from %s", original_scheduler_definition)
-            try:
-                if original_scheduler_definition.startswith("s3"):
-                    bucket_parsing_result = parse_bucket_url(original_scheduler_definition)
-                    result = AWSApi.instance().s3.get_object(
-                        bucket_name=bucket_parsing_result["bucket_name"], key=bucket_parsing_result["object_key"]
-                    )
-                    scheduler_definition = result["Body"].read().decode("utf-8")
-                elif original_scheduler_definition.startswith("https"):
-                    try:
-                        with urlopen(original_scheduler_definition) as f:  # nosec nosemgrep
-                            scheduler_definition = f.read().decode("utf-8")
-                    except Exception:
-                        raise ValidationError("The provided URL is invalid or unavailable.")
-                else:
-                    raise ValidationError(
-                        "The provided value for SchedulerDefinition is invalid. "
-                        "You can specify this as an S3 URL, HTTPS URL or as an inline YAML object."
-                    )
-                LOGGER.info("Using the following scheduler plugin definition:\n%s", scheduler_definition)
-                data["SchedulerDefinition"] = yaml.safe_load(scheduler_definition)
-            except YAMLError as e:
-                raise ValidationError(
-                    f"The retrieved SchedulerDefinition ({original_scheduler_definition}) is not a valid YAML."
-                ) from e
-            except Exception as e:
-                raise ValidationError(
-                    f"Error while downloading scheduler definition from {original_scheduler_definition}: {str(e)}"
-                ) from e
+            self._validate_scheduler_definition_url(original_scheduler_definition, s3_bucket_owner)
+            self._fetch_scheduler_definition_from_url(
+                original_scheduler_definition, s3_bucket_owner, scheduler_definition_checksum, data
+            )
+        elif s3_bucket_owner or scheduler_definition_checksum:
+            raise ValidationError(
+                "SchedulerDefinitionS3BucketOwner or SchedulerDefinitionChecksum can only specified when "
+                "SchedulerDefinition is a URL."
+            )
         return data
 
 
@@ -1498,15 +1670,15 @@ class SchedulingSchema(BaseSchema):
 
     scheduler = fields.Str(
         required=True,
-        validate=validate.OneOf(["slurm", "awsbatch"]),
+        validate=validate.OneOf(["slurm", "awsbatch", "plugin"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     # Slurm schema
-    slurm_settings = fields.Nested(SlurmSettingsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    slurm_settings = fields.Nested(SlurmSettingsSchema, metadata={"update_policy": UpdatePolicy.IGNORED})
     slurm_queues = fields.Nested(
         SlurmQueueSchema,
         many=True,
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP_ON_REMOVE, "update_key": "Name"},
     )
     # Awsbatch schema:
     aws_batch_queues = fields.Nested(
