@@ -1373,11 +1373,11 @@ class ComputeFleetConstruct(Construct):
                 subnet_id=queue.networking.subnet_ids[0],
             )
         ]
-        for device_index in range(1, compute_resource.max_network_interface_count):
+        for network_interface_index in range(1, compute_resource.max_network_interface_count):
             compute_lt_nw_interfaces.append(
                 ec2.CfnLaunchTemplate.NetworkInterfaceProperty(
-                    device_index=device_index,
-                    network_card_index=device_index,
+                    device_index=1,
+                    network_card_index=network_interface_index,
                     interface_type="efa" if compute_resource.efa and compute_resource.efa.enabled else None,
                     groups=queue_lt_security_groups,
                     subnet_id=queue.networking.subnet_ids[0],
