@@ -41,6 +41,19 @@ def get_partition():
     return next(("aws-" + partition for partition in ["us-gov", "cn"] if get_region().startswith(partition)), "aws")
 
 
+def get_url_domain_suffix():
+    """Get domain suffix."""
+    if get_partition() == "aws-cn":
+        return "amazonaws.com.cn"
+    else:
+        return "amazonaws.com"
+
+
+def replace_url_parameters(url):
+    """Replace ${Region} and ${URLSuffix} in url."""
+    return url.replace("${Region}", get_region()).replace("${URLSuffix}", get_url_domain_suffix())
+
+
 def generate_random_name_with_prefix(name_prefix):
     """
     Generate a random name that is no more than 63 characters long, with the given prefix.
@@ -81,7 +94,7 @@ def zip_dir(path):
 
     The archive is created in memory and a file handler is returned by the function.
     :param path: directory containing the resources to archive.
-    :return file handler pointing to the compressed archive.
+    :return: file handler pointing to the compressed archive.
     """
     file_out = BytesIO()
     with zipfile.ZipFile(file_out, "w", zipfile.ZIP_DEFLATED) as ziph:
@@ -119,7 +132,7 @@ def to_utc_datetime(time_in, default_timezone=datetime.timezone.utc) -> datetime
     be timestamps in UTC timezone.
     :param default_timezone: Timezone to assum in the event that the time is
     unspecified in the input parameter. This applies only for datetime and str inputs
-    :return time as a datetime in UTC timezone
+    :return: time as a datetime in UTC timezone
     """
     if isinstance(time_in, int):
         if time_in > 1e12:
@@ -142,7 +155,7 @@ def to_iso_timestr(time_in: datetime.datetime) -> str:
     Convert a given datetime ISO 8601 format with milliseconds.
 
     :param time_in: datetime to be converted
-    :return time in ISO 8601 UTC format with ms (e.g. 2021-07-15T01:22:02.655Z)
+    :return: time in ISO 8601 UTC format with ms (e.g. 2021-07-15T01:22:02.655Z)
     """
     if time_in.tzinfo is None:
         time_ = time_in.replace(tzinfo=datetime.timezone.utc)
@@ -258,7 +271,8 @@ def error(message) -> NoReturn:
 
 
 def get_cli_log_file():
-    return os.path.expanduser(os.path.join("~", ".parallelcluster", "pcluster-cli.log"))
+    default_log_file = os.path.expanduser(os.path.join("~", ".parallelcluster", "pcluster-cli.log"))
+    return os.environ.get("PCLUSTER_LOG_FILE", default=default_log_file)
 
 
 def ellipsize(text, max_length):
@@ -310,3 +324,12 @@ def load_json_dict(file_path):
         json_content = json.load(file)
 
     return json_content
+
+
+def get_attr(obj, attributes, default=None):
+    """Get nested object attribute and return default if attr does not exist."""
+    for attribute in attributes.split("."):
+        obj = getattr(obj, attribute, None)
+        if obj is None:
+            return default
+    return obj
